/**************************************************  Chip Functions Header  ***************************************************

	Red Rapids
	797 N Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

**************************************************  Chip Functions Header  ***************************************************/

#ifndef _CHIP_FUNCTIONS_H_
#define _CHIP_FUNCTIONS_H_

#if defined(__cplusplus)
	extern "C" {
#endif

// Structure to set the enable bit on the AD8000 amplifier chip.
typedef struct
{
	unsigned Enable[2];
} s_AD8000;

// Structure to store configuration register settings for the AD5628 DAC chip.
typedef struct
{
	unsigned Payload[8][2];
} s_AD5628;

// Structure to store configuration register settings for the AD9142A clock distribution chip.
typedef struct
{
	unsigned Payload[63][2];
} s_AD9142A;

// Structure to set the enable bit on the ADA4927 amplifier chip.
typedef struct
{
	unsigned Enable[2];
} s_ADA4927;

// Structure to store configuration register settings for the AD9512 clock distribution chip.
typedef struct
{
	unsigned Payload[22][2];
} s_AD9512;

// Structure to store configuration register settings for the AD9652 ADC chip.
typedef struct
{
	unsigned Payload[26][2];
} s_AD9652;

// Structure to store configuration register settings for the AD9653 ADC chip.
typedef struct
{
	unsigned Payload[23][2];
} s_AD9653;

// Structure to store configuration register settings for the ADC012D1600 ADC chip.
typedef struct
{
	unsigned Payload[11][2];
} s_ADC12D1600;

// Structure to store configuration settings for the ADF4110 synthesizer chip.
typedef struct
{
	unsigned A;
	unsigned B;
} s_ADF4110;

// Structure to set the two enable bits on the ADL5566 amplifier chip.
typedef struct
{
	unsigned Enable[2][2];
} s_ADL5566;

// Structure to store configuration register settings for the ADS42LB69 ADC chip.
typedef struct
{
	unsigned Payload[21][2];
} s_ADS42LB69;

// Structure to store configuration register settings for the LMK01000 clock distribution chip.
typedef struct
{
	unsigned Payload[10][2];
} s_LMK01000;

// Structure to store configuration register settings for the LTC1661 DAC chip.
typedef struct
{
	unsigned Payload[8][2];
} s_LTC1661;

// Structure to store configuration settings for any programmable synthesizer chip.
typedef struct
{
	unsigned Frequency;
	unsigned Payload[8][2];
} s_PROGSYNTH;

// Prototypes for each of the chip functions.
void ADA4927Write(s_Handle *p_HW, unsigned ChipSelect, unsigned PortNumber, unsigned Data);
void AD5628Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Command, unsigned Address, unsigned Data);
void AD8000Write(s_Handle *p_HW, unsigned ChipSelect, unsigned PortNumber, unsigned Data);
unsigned AD9142ARead(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void AD9142AWrite(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
unsigned AD9512Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void AD9512Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
void AD9652Cal(s_Handle *p_HW, unsigned ChipSelect);
unsigned AD9652Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void AD9652Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
unsigned AD9653Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void AD9653Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
void ADC12D1600Cal(s_Handle *p_HW, unsigned ChipSelect);
unsigned ADC12D1600Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void ADC12D1600Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
void ADL5566Write(s_Handle *p_HW, unsigned ChipSelect, unsigned PortNumber, unsigned Data);
unsigned ADS42LB69Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void ADS42LB69Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
unsigned FXL6408Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void FXL6408Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
unsigned INA3221Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void INA3221Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
void LMK01000Sync(s_Handle *p_HW, unsigned ChipSelect);
void LMK01000Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
void LTC1661Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);
unsigned LTC2991Read(s_Handle *p_HW, unsigned ChipSelect, unsigned Address);
void LTC2991Write(s_Handle *p_HW, unsigned ChipSelect, unsigned Address, unsigned Data);

#ifdef __cplusplus
}
#endif

#endif
