/*************************************************  Driver Functions Header  ***************************************************
	Red Rapids
	797 N Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*************************************************  Driver Functions  Header ***************************************************/

#ifndef _DRIVER_H_
#define _DRIVER_H_

#if defined(__cplusplus)
    extern "C" {
#endif

// Set the number of Red Rapids devices attached to the host.
#define NUM_DEVICES			1

// Structure to store descriptor information for UIO devices (/dev/uio#) that are identified as Red Rapids hardware.
typedef struct {
	unsigned UIONumber;				// Linux UIO character device file number.
	unsigned SubSysID;				// PCI subsystem ID.
	unsigned long long BAR0Size;	// Size of the BAR0 space.
	unsigned long long BAR2Size;	// Size of the BAR2 space.
} s_UIOScan;

// Structure to store mapping information for Red Rapids UIO character devices (/dev/uio#).
typedef struct {
	int fd;							// UIO file descriptor returned by open().
	volatile unsigned long long *BAR0;		// Pointer to BAR0 register space.
	volatile unsigned long long *BAR2;		// Pointer to BAR2 register space.
} s_DEVMap;

// Structure to store buffer information retrieved from Red Rapids DMA character devices (/dev/rrdma#).
typedef struct {
	int fd;							// DMA file descriptor returned by open().
	unsigned PageNumber;			// Number of the currently mapped DMA page starting from zero.
	unsigned PageCount;				// Number of DMA pages available in the buffer.
	unsigned PagesPerMark;			// Number of DMA pages to transfer between completion marker (status) flags.
	unsigned BurstSizeB;			// Size (bytes) of each burst transaction per DMA page.
	unsigned BurstCount;			// Number of burst transactions per DMA page.
	unsigned long long *Address;	// Variable used to store both physical and virtual addresses assigned to each DMA page.
} s_DMABuf;

// Structure to store the Red Rapids device hardware handle.
typedef struct {
	unsigned DevNum;				// Device number assigned to the Red Rapids hardware (zero based).
	s_DEVMap DEV;					// UIO device structure.
	s_DMABuf *DMA;					// DMA config for all DMA buffers.
	void *User;						// Undefined pointer available to user applications.
} s_Handle;

// Prototypes for each of the driver functions.
int DEVClose(s_UIOScan UIO, s_DEVMap *p_DEV);
void DEVInit(s_Handle *p_HW, unsigned DevNum);
int DEVOpen(s_UIOScan UIO, s_DEVMap *p_DEV);
int DMAClose(int fd);
int DMAFirstPage(s_DMABuf *p_DMA);
void DMAInit(s_DMABuf *p_DMA);
int DMAMap(s_DMABuf *p_DMA);
int DMAOpen(s_DMABuf *p_DMA, unsigned DeviceNumber, unsigned BufferIndex);
int DMAUnmap(s_DMABuf *p_DMA);
unsigned UIOScan(s_UIOScan *p_UIO);

#ifdef __cplusplus
}
#endif

#endif

